package po

import (
	"encoding/json"
	"fmt"
	"gitea.cdlsxd.cn/sdk/plugin/proto"
	"github.com/go-playground/validator/v10"
)

type Req interface {
	Validate() error
	GetReId() string
	ToJson() []byte
}

var _ Req = (*OrderReq)(nil)
var _ Req = (*QueryReq)(nil)
var _ Req = (*Notify)(nil)

func (req *OrderReq) Validate() error {
	err := validator.New().Struct(req)
	if err != nil {
		for _, err = range err.(validator.ValidationErrors) {
			return proto.ErrorParamFail(fmt.Sprintf("参数有误: %v", err))
		}
	}
	return nil
}

func (req *OrderReq) GetReId() string {
	return req.Qid
}

func (req *OrderReq) ToJson() []byte {
	b, _ := json.Marshal(req)
	return b
}

func (req *QueryReq) Validate() error {
	err := validator.New().Struct(req)
	if err != nil {
		for _, err = range err.(validator.ValidationErrors) {
			return proto.ErrorParamFail(fmt.Sprintf("参数有误: %v", err))
		}
	}
	return nil
}

func (req *QueryReq) GetReId() string {
	return req.TraceId
}

func (req *QueryReq) ToJson() []byte {
	b, _ := json.Marshal(req)
	return b
}

func (req *Notify) Validate() error {
	err := validator.New().Struct(req)
	if err != nil {
		for _, err = range err.(validator.ValidationErrors) {
			return proto.ErrorParamFail(fmt.Sprintf("参数有误: %v", err))
		}
	}
	return nil
}

func (req *Notify) GetReId() string {
	return req.Headers.ReqId
}

func (req *Notify) ToJson() []byte {
	b, _ := json.Marshal(req.Body)
	return b
}
