package internal

import (
	"context"
	"encoding/json"
	"fmt"
	"gitea.cdlsxd.cn/sdk/plugin/proto"
	"github.com/stretchr/testify/assert"
	"net/http"
	"testing"
)

var srv = &ZLTXV1Service{}

func config() []byte {
	c := &Config{
		AppId:      "1",
		AppKey:     "1e2bf7a04b8b1e6be5dc78d04e8639c9",
		BaseUri:    "http://test.openapi.1688sup.cn",
		NotifyUrl:  "https://gateway.dev.cdlsxd.cn/yxh5api/v1/order/direct/notify",
		MerchantId: 25537,
	}
	//c := &Config{
	//	AppId:      "101",
	//	AppKey:     "95E7EC7D4A394FF8D11788E5E436DE99",
	//	BaseUri:    "https://openapi.1688sup.com",
	//	NotifyUrl:  "https://gateway.dev.cdlsxd.cn/yxh5api/v1/order/direct/notify",
	//	MerchantId: 25715,
	//}
	marshal, _ := json.Marshal(c)
	return marshal
}

func TestConfig(t *testing.T) {
	t.Run("TestConfig", func(t *testing.T) {
		c := config()
		fmt.Printf("%s\n", string(c))
		assert.NotEmpty(t, c)
	})
}

func TestOrder(t *testing.T) {
	request := &proto.OrderRequest{
		Config: config(),
		Order: &proto.OrderRequest_Order{
			OrderNo:  "test_plugin_zltx_v1_direct_2",
			Account:  "583989020@qq.com",
			Quantity: 1,
			Extra:    nil,
		},
		Product: &proto.OrderRequest_Product{
			ProductNo: "101",
			Extra:     []byte(`{}`),
		},
	}

	t.Run("TestOrder", func(t *testing.T) {
		got, err := srv.Order(context.Background(), request)
		if err != nil {
			t.Errorf("Order() error = %v", err)
			return
		}
		t.Logf("%+v\n", got)
		assert.Equal(t, int(proto.Status_ING), int(got.Result.Status))
	})
}

func TestQuery(t *testing.T) {
	request := &proto.QueryRequest{
		Config: config(),
		Order: &proto.QueryRequest_Order{
			OrderNo: "test_plugin_zltx_v1_direct_2",
			TradeNo: "",
			Account: "",
			Extra:   nil,
		},
	}
	t.Run("TestQuery", func(t *testing.T) {
		got, err := srv.Query(context.Background(), request)
		if err != nil {
			t.Errorf("Query() error = %v", err)
			return
		}
		t.Logf("%+v\n", got)
		assert.Equal(t, int(proto.Status_SUCCESS), int(got.Result.Status))
	})
}

func TestNotify(t *testing.T) {
	in := &proto.NotifyRequest{
		Config:  config(),
		Queries: nil,
		Headers: []byte(`{"Accept-Encoding":["gzip, deflate, br"],"Authorization":["MD5 appid=1,sign=6EF1284331EAC9734715C87FEF59D2EF"],"Connection":["close"],"Content-Length":["146"],"Content-Type":["application/json"],"Cookie":[""],"User-Agent":["GuzzleHttp/6.5.5 curl/7.69.1 PHP/7.2.34"],"X-Remoteaddr":["172.21.0.1"]}`),
		Body:    []byte(`{"merchantId":25537,"outTradeNo":"test_plugin_zltx_v1_direct_2","tradeNo":"716586208482959361","status":"03","rechargeAccount":"583989020@qq.com"}`),
	}
	t.Run("TestNotify", func(t *testing.T) {
		got, err := srv.Notify(context.Background(), in)
		if !assert.Nil(t, err) {
			t.Errorf("Notify() error = %v", err)
			return
		}

		responseHeaders := make(http.Header)
		err = json.Unmarshal([]byte(got.Headers), &responseHeaders)
		if err != nil {
			t.Error(err)
			return
		}
		for s, strings := range responseHeaders {
			t.Logf("%s: %+v", s, strings)
		}

		fmt.Printf("%+v\n", got)
		assert.Equal(t, int(proto.Status_SUCCESS), int(got.Result.Status))
	})
}
